#!/bin/sh
# usage: . cron_fns.sh
# loads common cronfile manipulation functions

GA_CRONFILE="$GA_CONF/.feedcron"


###########################################################################
# load_cron will install the .feedcron file in crontab if it exists
###########################################################################
load_cron() {
    # Don't modify crontab if not correct account.
    if [ "X$GA_PRODUCTION_USER" != "X" ]
    then
        GA_whoami
        if [ "$GA_PRODUCTION_USER" != "$GA_USERNAME" ]
        then
            return 0
        fi
    fi

    # return 0 normally, or 1 if the cron is changed
    rc=0

    # Always make sure proper cron entry exists if configured to run.
    if [ -f $GA_CRONFILE ]
    then
        cnt=`wc -l $GA_CRONFILE | awk '{print $1}'`
        if [ $cnt -ne 1 ]
        then
            # .feedcron required to have one line
            return 1
        fi

        crontab -l >/dev/null 2>&1
        if [ $? -eq 0 ]
        then
            crontab -l | egrep -v '(# DO NOT EDIT|# \()' > $TMPFILE
            # If nwfdon or amxstart exists there is no more to do.
            # NOTE: amxstart is used for interceiver installations
            tmp=`head -1 $GA_CRONFILE`
            egrep '(nwfdon|amxstart)' $TMPFILE >/dev/null 2>&1
            if [ $? -ne 0 ]
            then
                cat $GA_CRONFILE >> $TMPFILE
                crontab $TMPFILE
                rc=1
            fi
	    rm -r $TMPFILE
        else
            # Just submit cronfile alone.
            crontab $GA_CRONFILE
            rc=1
        fi
    fi

    return $rc
}
