# This file contains useful defines and routines needed by two or
# more bourne shell scripts.
# $Id: gautil.sh 773059 2014-10-06 18:12:01Z rpmbuild $
# $URL: http://subversion.mmu.acquiremedia.com/svn/amc/msrc/bin/gautil/tags/gautil-RelS9.2-3/src/gautil.sh $

#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
# IMPORTANT NOTE!
#
# All routines defined in this file must start with GA_ folowed
# by lower case letters in order to easily identify them.
# All variables defined must start with GA_ followed by all
# upper case letters.
#
#=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

# Global definition section

GA_NWSYS=/nwsys
GA_ROOT=/nwsys/release
GA_BACKUP=/nwbackup
GA_CONF="$GA_ROOT/conf"
GA_BIN="$GA_ROOT/bin"
GA_AOS="$GA_ROOT/aos"
GA_ETC="$GA_ROOT/etc"
GA_LOGS="$GA_ROOT/logs"
GA_TMP="$GA_ROOT/tmp"
GA_RAM="/ram0"
GA_INDEX="/nwsys/index"
GA_TEXT="/nwtext"
GA_CONF_SYS=$GA_CONF/system
GA_SYSTEM_DOT_CONF=$GA_CONF/system.conf
GA_USERS_ACC=$GA_CONF_SYS/accounts.users
GA_OFFSET_CONF=$GA_CONF_SYS/offsets.conf
GA_NEWSPAR_CONF=$GA_CONF_SYS/newspar.conf
GA_SLAM_CONF=$GA_CONF_SYS/slam.conf
GA_S_DISK=/nwsys/s_disk
GA_QSERVE="/nwsys/qserve"

GA_SYSCFG="$GA_ROOT/conf/.syscfg"
GA_SYSDEFS="$GA_ROOT/bin/sysdefs"
GA_WIRES="$GA_ROOT/conf/wires.conf"
GA_TMP="$GA_ROOT/tmp"
GA_RELEASE_FILE="$GA_CONF/release"
GA_CRONFILE="$GA_CONF/.feedcron"
GA_LOGMSG="$GA_BIN/logmsg"
GA_WARN=FALSE
GA_HATCHD_IS_UP=TRUE
GA_INDEXING_FILE="$GA_TMP/indexing"
GA_UPTIME_FILE="$GA_TMP/uptime"
GA_LIB_DIR="/nwsys/release/lib"
GA_LIB_DIR_64="/nwsys/release/lib64"

CONFIG_FILES="$GA_SYSCFG $GA_RELEASE_FILE"
ENV_FILE="$GA_CONF_SYS/ServerEnv.sh"
ENV_FILE_TMP="$GA_CONF_SYS/ServerEnv.sh.$$"

WHOAMI=/usr/ucb/whoami
WHOAMI_LINUX=/usr/bin/whoami
TMPFILE=/tmp/tmp.$$
TMPFILE1=/tmp/tmp1.$$

ENV_MARKER="#XxXx_End_Of_File_XxXx"

GA_WCMD="/usr/wlabs/wlabscmd"

###########################################################################
# GA_list_contains
# Compare all elements in the string passed as arg1 to arg2.  Return
# 1 if match is found else return 0.
#
# Usage: GA_list_contains match "string of words"
###########################################################################
GA_list_contains () {
    for tmp in $2
    do
        if [ $tmp = "$1" ]
        then
            return 1
        fi
    done
    return 0
}

###########################################################################
# GA_file_contains
# Determine if 'match on' argument is contained in the file 'filename'.
# Return 1 if match is found else return 0.
#
# Usage: GA_file_contains "match on" filename
###########################################################################
GA_file_contains () {
    if grep "$1" $2 >/dev/null
    then
        return 1
    else
        return 0
    fi
}

###########################################################################
# Setup variables and determine operating system type.
# Pass REBUILD to force refresh of environment variables.
#
# Usage: GA_init [REBUILD]
###########################################################################
GA_init () {
    if [ ! -f $GA_SYSCFG ]
    then
        echo "WARNING: system configuration file is missing"
        exit 1
    fi

    if [ ! -f $GA_SYSDEFS ]
    then
        echo "WARNING: system defaults 'sysdefs' is missing"
        exit 1
    fi

    if [ "X$1" = "XREBUILD" ]
    then
        rm -f $ENV_FILE>/dev/null 2>&1
        if [ $? -ne 0 ]
        then
            echo "WARNING: $ENV_FILE could not be deleted; check permissions"
            exit 1
        fi
    fi

    #
    # Make sure the LD_LIBRARY_PATH contains the necessary paths for
    # shared libraries.
    # NOTE: If defined in system.conf use that as an override.
    #
    LD_LIBRARY_PATH=`grep '^LD_LIBRARY_PATH:' $GA_SYSTEM_DOT_CONF | awk '{print $2}'`
    if [ "X$LD_LIBRARY_PATH" = "X" ]
    then
        if [ `uname` = "SunOS" ]
        then
            LD_LIBRARY_PATH="/nwsys/release/lib:/usr/openwin/lib:/opt/SUNWspro/lib:/opt/SUNWconn/lib:/usr/ucblib:/usr/local/lib:/lib:/usr/lib"
        else
            LD_LIBRARY_PATH="/nwsys/release/lib:/lib:/usr/lib:/usr/local/lib:/usr/X11R6/lib"
        fi
    fi
    export LD_LIBRARY_PATH

    setup_env

    for proc in $GA_ROOT_PROCS
    do
        if [ -f $GA_ETC/$proc ]
        then
            $GA_WCMD chown root $GA_ETC/$proc
            $GA_WCMD chmod 4555 $GA_ETC/$proc
        fi
    done
}

#################################################################
# Query for yes or no response
#################################################################
GA_yesno() {
    while printf "   $*""? (y/n) "; read f_yn
    do
        case "$f_yn" in
            [Yy] | yes | YES | Yes) return 0 ;;
            [Nn] | no | NO | No) return 1 ;;
            *) printf "\nPlease Enter 'y' or 'n' " ;;
        esac
    done
}

###########################################################################
# Retrieve list of process id's for all process with name matching
# arg1. List is stored in global variable GA_PIDLIST.  GA_init must
# be called prior to this function.
#
# Usage: GA_load_pidlist "process_name_to_match"
###########################################################################
GA_load_pidlist() {
    if [ "$GA_OPR_SYS" = "Solaris" -o "$GA_OPR_SYS" = "Linux" ]
    then
        GA_PIDLIST="`$GA_PS | grep -w $1 | awk '{print $1}' | sort -n`"
    elif [ "$GA_OPR_SYS" = "HPUX" ]
    then
        # Since HP doesn't support the -w switch, we must explicitly
        # filter out 'f$1' so dvnews doesn't match fdvnews.
        GA_PIDLIST="`$GA_PS | egrep $1 | egrep -v f$1 | awk '{print $1}' | sort -n`"
    else
        # SunOs default case, also filter out 'f$1' like HP
        GA_PIDLIST="`$GA_PS | grep $1 | egrep -v '(egrep|f$1)' | awk '{print $2}' | sort -n`"
    fi
}

###########################################################################
# Retrieve list of process id's for all child processes belonging to
# arg1. List is stored in global variable GA_PIDLIST.  GA_init must
# be called prior to this function.  Caller is responsible for handling
# empty GA_PIDLIST.
#
# Usage: GA_load_child_pidlist "parent_process_name"
###########################################################################
GA_load_child_pidlist() {
    # Load all pids under this name
    GA_load_pidlist $1
    if [ "X$GA_PIDLIST" = "X" ]
    then
        return
    fi

    proc_list=$GA_PIDLIST

    # Determine parent process PID.  For now we will go with the
    # smallest PID.
    rm -f $TMPFILE >/dev/null 2>&1
    last_pid=99999
    for procid in $proc_list
    do
        if [ $procid -lt $last_pid ]
        then
            echo "$procid" > $TMPFILE
            last_pid=$procid
        fi
    done
    parent_pid=`cat $TMPFILE`

    # Now find all child processes of this PID.  Note that we handle
    # the fact that child processes may be running with different names.
    rm -f $TMPFILE1 >/dev/null 2>&1
    $GA_PSEF > $TMPFILE
    while read uid_num pid_num ppid_num rest_of_line
    do
        if [ "X$ppid_num" = "X$parent_pid" ]
        then
            echo "$pid_num" >> $TMPFILE1
        fi
    done < $TMPFILE

    if [ -f $TMPFILE1 ]
    then
        GA_PIDLIST=`cat $TMPFILE1`
    else
        GA_PIDLIST=""
    fi
    rm -f $TMPFILE $TMPFILE1 >/dev/null 2>&1
}

###########################################################################
# Determine user name of executing process regardless of platform
# we are running on.  The user name is stored in variable GA_USERNAME
#
# Usage: GA_whoami
###########################################################################
GA_whoami () {

    if [ -x $WHOAMI ]
    then
        GA_USERNAME=`$WHOAMI`
    elif [ -x $WHOAMI_LINUX ]
    then
        GA_USERNAME=`$WHOAMI_LINUX`
    else
        # This should work across all platforms
        touch $TMPFILE
        GA_USERNAME=`ls -l $TMPFILE | awk '{print $3}'`
        rm $TMPFILE
    fi
}

###########################################################################
# Verify that user account is correct if this server is configured
# as production.  NOTE: This routine will log error and exit if this
# check fails.
#
# Usage: GA_verify_production_user caller_logname
###########################################################################
GA_verify_production_user () {

    # If this is a production server check that the production user matches
    if [ "X$GA_PRODUCTION_USER" != "X" ]
    then
        GA_whoami
        if [ "$GA_PRODUCTION_USER" != "$GA_USERNAME" ]
        then
            printf "CRITICAL: production server; must be $GA_PRODUCTION_USER\n"
            $GA_LOGMSG $1 CRIT "production server; must be $GA_PRODUCTION_USER"
            exit 1
        fi
        # Extra check to make sure that shell was started with the
        # production user's startup files.  Prevents system from being
        # started after an su nadmin (without using the '-')
        # USER variable will be loaded by the shell.
        if [ "X$USER" != "X" ]
        then
            myname=$USER
        else
            # HOME should owned by the USER.
            myname=`ls -ld $HOME | awk '{print $3}'`
        fi
        if [ "$myname" != "$GA_PRODUCTION_USER" ]
        then
            printf "CRITICAL: production server; USER is currently '$myname'\n"
            printf "CRITICAL: production server; use su - $GA_PRODUCTION_USER\n"
            $GA_LOGMSG $1 WARN "production server; use su - $GA_PRODUCTION_USER"
            exit 1
        fi
    fi
}

###########################################################################
# See if passed in process name should be linked to different executable.
# If is should be, verify checksum and perform appropriate update if
# different.
#
# Usage: GA_check_for_link process_name
###########################################################################
GA_check_for_link () {
    # Link: link-from link-to1 link-to2 ...
    # look for $1 in any of the link-tos, return the link-from name
    # need to match a whole word, to avoid double matcheslike this (for metabotd):
    #    Link: metabotd.42 metabotd
    #    Link: metabotd.chitel mbotdc
    # solution: add a space to the end of the line, and match on " $1 "
    LinkName=`sed -n -e '/^Link:/s/$/ /p' $GA_SYSCFG | grep " $1 " | awk '{print $2}'`

    if [ "X$LinkName" = "X" ]
    then
        return
    fi

    # Verify checksum of linked daemon.
    if [ ! -f $GA_ETC/$LinkName ]
    then
        printf "\nWARNING: $LinkName does not exist - cannot start $1\n"
        exit 1
    fi

    # If the target process doesn't exist, just touch temp
    # file for now.  It will be removed and updated with the
    # real one.
    if [ ! -f $GA_ETC/$1 ]
    then
        touch $GA_ETC/$1
    fi

    # Verify that the two processes share the same inode (i.e. are linked).

    link_name_inode=`stat -c %i $GA_ETC/$LinkName`
    proc_inode=`stat -c %i $GA_ETC/$1`

    if [ $proc_inode -ne $link_name_inode ]
    then
        printf "    inode mismatch: re-linking $LinkName -> $1\n"

        chmod +w $GA_ETC/$1 >/dev/null
        rm -f $GA_ETC/$1

        # HARD LINK
        ln $GA_ETC/$LinkName $GA_ETC/$1
        if [ $? -ne 0 ]
        then
            printf "\nWARNING: Failed to link $LinkName -> $1\n"
            exit 1
        fi
    fi
}

GA_renice () {
    Procname=$1

    PRI="`grep ^Renice: $GA_SYSCFG | grep $Procname | awk '{print $3}'`"
    if [ "X$PRI" = "X" ]
    then
        return 0
    fi

    if [ "$GA_OPR_SYS" != "SunOs" ]
    then
        pidlist="`$GA_PS | egrep $Procname | awk '{print $1}'`"
    else
        pidlist="`$GA_PS | egrep $Procname | grep -v egrep | awk '{print $2}'`"
    fi

    if [ "X$pidlist" = "X" ]
    then
        printf "\nWARNING: renice failed - cannot retrieve PID for $Procname\n"
        return 1
    fi

    if [ ! -x $GA_WCMD ]
    then
        printf "\nWARNING: wlabscmd does not exist. Cannot renice $Procname\n"
        return 1
    fi

    printf "renice to $PRI ... "

    $GA_WCMD renice $PRI $pidlist>/dev/null
    if [ $? -ne 0 ]
    then
        printf "\nWARNING: Failed to change priority for $Procname ($pidlist)\n"
        return 1
    fi

    return 0
}

###########################################################################
# GA_rebuild_env
# Delete and recreate from scratch the ServerEnv file.
#
# Usage: GA_rebuild_env
###########################################################################
GA_rebuild_env () {

    rm -f $ENV_FILE>/dev/null 2>&1
    if [ $? -ne 0 ]
    then
        echo "WARNING: $ENV_FILE could not be deleted; check permissions"
        exit 1
    fi

    setup_env
    return 0
}

#################################################################
# Only locally used routines from here down.
#################################################################

# This routine returns 1 if the environment file is not present
# of is older that the configuration files required to set the
# variable.  If the environment file is found to be usable, source
# the file to set variables and return 0.
check_env_file () {

    if [ ! -f $ENV_FILE ]
    then
        touch $ENV_FILE
        return 1
    else
        # Verify integrity of environment file.  Last line should always
        # have the ENV_MARKER.
        last=`tail -1 $ENV_FILE`
        if [ "X$last" != "X$ENV_MARKER" ]
        then
            rm -f $ENV_FILE
            touch $ENV_FILE
            return 1
        fi

        if [ -f $GA_SYSTEM_DOT_CONF ]
        then
            CONFIG_FILES="$CONFIG_FILES $GA_SYSTEM_DOT_CONF"
        fi
        if [ -f $GA_NEWSPAR_CONF ]
        then
            CONFIG_FILES="$CONFIG_FILES $GA_NEWSPAR_CONF"
        fi
        # ENV_FILE must be newer than .syscfg, system.conf and newspar.conf
        newest_file=`ls -t $ENV_FILE $CONFIG_FILES | head -1`
        if [ "$newest_file" != "$ENV_FILE" ]
        then
            rm -f $ENV_FILE
            touch $ENV_FILE
            return 1
        fi
    fi

    # If the environment file is zero length, we must return 1 so
    # that it will be created from scratch.  This can happen if we
    # run out of disk space.  An appropriate error will be logged when
    # this is true in the setup env routine below.
    if [ ! -s $ENV_FILE ]
    then
        return 1
    fi

    # File exists and it newer that appropriate config files,
    # so just source it.
    . $ENV_FILE
    return 0
}

# Try to use pre-defined environment file if possible.  If we cannot,
# set all variables now and write a new environment file for all future
# use.  IMPORTANT NOTE:  Any variables being added to this routine must
# have corresponding call to add_to_file()
setup_env () {

    # Always load the uptime file to avoid having to rebuild entire env.
    if [ -f $GA_UPTIME_FILE ]
    then
        GA_UPTIME="`cat $GA_UPTIME_FILE`" 
    else
        GA_UPTIME="NA"
    fi

    # Use saved environment file if possible.
    check_env_file
    if [ $? -eq 0 ]
    then
        return
    fi

    # Determine operating system type.  NOTE we must call add_to_file
    # after GA_ECHO is defined since it is needed.
    GA_OPR_SYS=`egrep '^Operating_system:' $GA_SYSCFG | awk '{print $2}'`

    if [ "$GA_OPR_SYS" = "SunOs" ]
    then
        GA_PS="ps -auxw"
        GA_PSEF="ps -auxw"
        GA_ECHO="/usr/5bin/echo"
        GA_DF="df"
    else
        GA_OPR_SYS_VER=`uname -r | sed 's/smp//'`
        GA_PS="ps -e"
        GA_PSEF="ps -ef"
        GA_ECHO=echo
        GA_DF="df -k"
    fi

    if [ "$GA_OPR_SYS" = "HPUX" ]
    then
        GA_OPR_SYS_VER=`uname -a | awk '{print substr($3,3,2)}'`
        GA_DF="bdf"
    fi

    add_to_file "GA_OPR_SYS" "$GA_OPR_SYS"
    add_to_file "GA_OPR_SYS_VER" "$GA_OPR_SYS_VER"
    add_to_file "GA_PS" "$GA_PS"
    add_to_file "GA_PSEF" "$GA_PSEF"
    add_to_file "GA_ECHO" "$GA_ECHO"
    add_to_file "GA_DF" "$GA_DF"

    GA_file_contains "^Backup_device:" $GA_SYSCFG
    if [ $? -eq 0 ]
    then
        printf "WARNING: system configuration file must specify Backup_device:\n"
        rm -f $ENV_FILE
        exit 1
    fi

    # Determine which types of servers are running on this system.
    gtype="`grep '^SERVER_TYPE:' $GA_SYSCFG | awk '{print $2,$3,$4,$5}'`"
    # Remove white space.
    GA_SERVER_TYPE=`echo $gtype`
    add_to_file "GA_SERVER_TYPE" "$GA_SERVER_TYPE"

    # Determine which type of server build configuration (caching/xagent)
    gtype=`$GA_SYSDEFS | grep SERVER_BLD_CFG: | awk '{print $2}'`
    # Remove white space.
    GA_SERVER_BLD_CFG=`echo $gtype`
    add_to_file "GA_SERVER_BLD_CFG" "$GA_SERVER_BLD_CFG"

    # Determine news engine configuration
    GA_SITENAME=`egrep '^Site_name:' $GA_SYSCFG | awk '{print $2}'`
    add_to_file "GA_SITENAME" "$GA_SITENAME"

    GA_RELEASE="`head -1 $GA_RELEASE_FILE`"
    add_to_file "GA_RELEASE" "$GA_RELEASE"

    GA_LOGGER=`egrep '^Logging_process:' $GA_SYSCFG | awk '{print $2}'`
    add_to_file "GA_LOGGER" "$GA_LOGGER"

    GA_PRIMARY=`egrep '^Primary_process:' $GA_SYSCFG | awk '{print $2}'`
    add_to_file "GA_PRIMARY" "$GA_PRIMARY"

    GA_SYS_TYPE=`egrep '^System_type:' $GA_SYSCFG | awk '{print $2}'`
    add_to_file "GA_SYS_TYPE" "$GA_SYS_TYPE"

    # NOTE when finding all Process omit the ':' to include specical
    # cases such as Process_news_dqer:
    GA_SECONDARY=`egrep '^Process' $GA_SYSCFG | awk '{print $2}'`
    add_to_file "GA_SECONDARY" "$GA_SECONDARY"

    GA_ALL="$GA_LOGGER $GA_PRIMARY $GA_SECONDARY"
    add_to_file "GA_ALL" "$GA_ALL"
    GA_ALL_PROCS="$GA_PRIMARY $GA_SECONDARY"
    add_to_file "GA_ALL_PROCS" "$GA_ALL_PROCS"
    GA_FEED_HANDLER=`egrep '^Feed_handler:' $GA_SYSCFG | awk '{print $2}'`
    add_to_file "GA_FEED_HANDLER" "$GA_FEED_HANDLER"
    GA_FEED_HANDLER_QUOT=`egrep '^Feed_handler_quotes:' $GA_SYSCFG | \
    awk '{print $2}'`
    add_to_file "GA_FEED_HANDLER_QUOT" "$GA_FEED_HANDLER_QUOT"
    GA_FEED_PROC=`egrep '^Feed_process:' $GA_SYSCFG | awk '{print $2}'`
    add_to_file "GA_FEED_PROC" "$GA_FEED_PROC"
    GA_ALL_FEEDS="$GA_FEED_HANDLER $GA_FEED_HANDLER_QUOT $GA_FEED_PROC"
    add_to_file "GA_ALL_FEEDS" "$GA_ALL_FEEDS"

    # Store production user if defined.
    GA_PRODUCTION_USER=`egrep '^PRODUCTION_USER' $GA_SYSCFG | awk '{print $2}'`
    add_to_file "GA_PRODUCTION_USER" "$GA_PRODUCTION_USER"

    # Store root procs if defined
    GA_ROOT_PROCS="`egrep '^RunAsRoot:' $GA_SYSCFG | awk '{print $2,$3,$4,$5,$6,$7,$8}'`"
    if [ "X$GA_ROOT_PROCS" != "X" ]
    then
        if [ ! -x $GA_WCMD ]
        then
            printf "\nWARNING: wlabscmd does not exist. Cannot run as root\n"
        else
            add_to_file "GA_ROOT_PROCS" "$GA_ROOT_PROCS"
        fi
    fi

	GA_JEMALLOC_PROCS="`egrep '^RunAsJeMalloc:' $GA_SYSCFG | awk '{print $2,$3,$4,$5,$6,$7,$8}'`"
    add_to_file "GA_JEMALLOC_PROCS" "$GA_JEMALLOC_PROCS"

    # Format the GA_PSE_CMD for all defined processes.  This includes
    # logger, and feeds.
    GA_PSE_CMD="$GA_PS | egrep '("
    first="TRUE"
    for gaproc in $GA_ALL $GA_ALL_FEEDS
    do
        if [ "$first" = "FALSE" ]
        then
            GA_PSE_CMD="${GA_PSE_CMD}|"
        fi
        GA_PSE_CMD="${GA_PSE_CMD}$gaproc"
        first="FALSE"
    done
    GA_PSE_CMD="${GA_PSE_CMD})'"
    add_to_file "GA_PSE_CMD" "$GA_PSE_CMD"

    GA_BCKUP_DEV=`egrep '^Backup_device:' $GA_SYSCFG | awk '{print $2}'`
    GA_BCKUP_FILE_PARTITION=`egrep '^Backup_file_partition:' $GA_SYSCFG | awk '{print $2}'`
    add_to_file "GA_BCKUP_DEV" "$GA_BCKUP_DEV"
    GA_BCKUP_FILE_PARTITION=`egrep '^Backup_file_partition:' $GA_SYSCFG | \
    awk '{print $2}'`
    add_to_file "GA_BCKUP_FILE_PARTITION" "$GA_BCKUP_FILE_PARTITION"

    GA_list_contains NEWS "$GA_SERVER_TYPE"
    if [ $? -eq 1 ]
    then
        # Set number of s_disk and r_disk processes that should be running
        GA_S_DISK_NUM=`$GA_SYSDEFS -v | grep S_DISK: | awk '{print $2}'`
        add_to_file "GA_S_DISK_NUM" "$GA_S_DISK_NUM"

        GA_R_DISK_NUM=`$GA_SYSDEFS -v | grep R_DISK: | awk '{print $2}'`
        add_to_file "GA_R_DISK_NUM" "$GA_R_DISK_NUM"

        GA_PAMD_NUM=`$GA_SYSDEFS -v | grep SIMUL_PAM_SERVERS | awk '{print $3}'`
        add_to_file "GA_PAMD_NUM" "$GA_PAMD_NUM"

        GA_PROFD_NUM=`$GA_SYSDEFS -v | grep SIMUL_PROFD_SERVERS | awk '{print $3}'`
        add_to_file "GA_PROFD_NUM" "$GA_PROFD_NUM"
    fi

    GA_list_contains QUOTES "$GA_SERVER_TYPE"
    if [ $? -eq 1 ]
    then
        # For now the number of quotd processes is always 2.
        GA_QUOTD_NUM=2
        add_to_file "GA_QUOTD_NUM" "$GA_QUOTD_NUM"

        GA_DQTICK_NUM=`egrep '^quote_mode:' $GA_SYSTEM_DOT_CONF`
        GA_list_contains non-interactive "$GA_DQTICK_NUM"
        if [ $? -eq 1 ]
        then
            GA_DQTICK_NUM=1
        else
            GA_DQTICK_NUM=2
        fi
        add_to_file "GA_DQTICK_NUM" "$GA_DQTICK_NUM"
    fi

    if [ -f $GA_SYSTEM_DOT_CONF ]
    then
        if grep '^garp_interval' $GA_SYSTEM_DOT_CONF >/dev/null 2>&1
        then
            GA_GARP_INT=`egrep '^garp_interval:' $GA_SYSTEM_DOT_CONF | awk '{print $2}'`
            if [ $GA_GARP_INT -eq 0 ]
            then
                GA_GARP_INT=8
            fi
        else
            GA_GARP_INT=8
        fi
        add_to_file "GA_GARP_INT" "$GA_GARP_INT"
    fi

    # Quick check here that the environment file is not 0 length.
    # This can happen if we are out of disk space.
    if [ ! -s $ENV_FILE_TMP ]
    then
        printf "WARNING: Cannot create $ENV_FILE\n"
        printf "WARNING: Check disk space immediately!\n"
    fi

    # Set perms to 777 in case this file is being created by a non
    # nadmin process that wants to take advantage of the defines
    # contained here.
    chmod 777 $ENV_FILE_TMP

    if [ "X$GA_OPR_SYS" = "XSolaris" -a "X$GA_OPR_SYS_VER" = "X5.6" ]
    then
        # Verify that for Solaris 2.6 (SunOS 5.6) the release has been
        # built on a Solaris 2.6 (SunOS 5.6) machine.
        BuildVer=`$GA_SYSDEFS -v | grep COMPILATION: | awk '{print $2}'`
        if [ "X$GA_OPR_SYS_VER" != "X$BuildVer" ]
        then
            printf "\n"
            printf "***************************************\n"
            printf "WARNING: Compilation version is not 5.6\n"
            printf "         Please contact software vendor\n"
            printf "***************************************\n"
            printf "\n"
            rm -f $ENV_FILE_TMP
            exit 1
        fi
    fi

    # The last line must always be ENV_MARKER to be used for integrity
    # checking.
    printf "$ENV_MARKER\n" >>$ENV_FILE_TMP

    mv -f $ENV_FILE_TMP $ENV_FILE
}

# Add variable to the env_file.
add_to_file () {
    printf "$1=\"$2\"\n">>$ENV_FILE_TMP
}


###########################################################################
# standard, non-printing check to see if a process is running
# returns 0 if running, 1 if not
###########################################################################
chk_process() {
    GA_load_pidlist $1

    if [ "X$GA_PIDLIST" != "X" ]
    then
        return 0
    else
        return 1
    fi
}

###########################################################################
# check_all_procs returns 0 if all feed procs are running
# or 1 otherwise.  Use to decide whether to update crontab or not
###########################################################################
check_all_procs() {
    for feedname in $GA_ALL_FEEDS
    do
        chk_process $feedname>/dev/null
        if [ $? -ne 0 ]
        then
            # As soon as we find a process that is down, we don't
            # want to install the crontab entry.  Just return.
            return 1
        fi
    done

    return 0
}


###########################################################################
#
# AlertSys API Utils
#
###########################################################################

if [ "X$ALERTSYS_DEV" = "X" ]
then
	ALERTSYS_BASE='http://nsyslog1.mmu.acquiremedia.com/alertsys'
else
	ALERTSYS_BASE='http://nsyslog1.mmu.acquiremedia.com/alertsys_dev'
fi

###########################################################################
#
# AlertSys_alert()
#
#  REQUIRED
#    -event EVENT
#    -data FILE
#
#  OPTIONAL
#    -program PROGRAM  (default: basename $0)
#    -title TITLE      (default: first line of data)
#    -ip IP
#    -created "YYYY-MM-DD HH:MM:SS"
#
#	returns 0 on success
#   returns 1 or 2 on error + stderr msg
#
###########################################################################
AlertSys_alert() {
	a_event=''
	a_data=''
	a_program=''
	a_title=''
	a_ip=''
	a_created=''

	while [ $# -gt 0 ]
	do
		case "$1" in
			-event)
				a_event=$2
				shift; shift;;
			-data)
				a_data=$2
				shift; shift;;
			-program)
				a_program=$2
				shift; shift;;
			-title)
				a_title=$2
				shift; shift;;
			-ip)
				a_ip=$2
				shift; shift;;
			-created)
				a_created=$2
				shift; shift;;
			*)
				echo "AlertSys_alert(): invalid option $1" >&2
				return 2;;
		esac
	done

	if ! which curl >/dev/null 2>/dev/null
	then
		echo "AlertSys_alert(): curl not found" >&2
		return 2
	fi

	if [ "X$a_event" = "X" ]
	then
		echo "AlertSys_alert(): missing -event" >&2
		return 2
	fi

	if [ "X$a_data" = "X" ]
	then
		echo "AlertSys_alert(): missing -data" >&2
		return 2
	fi

	if [ ! -f $a_data ]
	then
		echo "AlertSys_alert(): data file not found: $a_data" >&2
		return 2
	fi

	if [ "X$a_program" = "X" ]
	then
		a_program=`basename "$0"`
	fi

	if [ "X$a_title" = "X" ]
	then
		a_title=`head -1 $a_data`
	fi

	curl --silent --show-error \
		--output /tmp/alertsys_out.$$ \
		--url "$ALERTSYS_BASE/alerts/add/" \
		-F event="$a_event" \
		-F alert_data="<$a_data" \
		-F program="$a_program" \
		-F title="$a_title" \
		-F ip="$a_ip" \
		-F created="$a_created"

	if [ $? -ne 0 ]
	then
		echo "AlertSys_alert(): curl command failed" >&2
		return 1
	fi

	a_output=`cat /tmp/alertsys_out.$$`

	if [ "X$a_output" != "XOK" ]
	then
		echo "AlertSys_alert(): alertsys server failed to save alert"  >&2
		echo "AlertSys_alert(): alertsys server response: /tmp/alertsys_out.$$" >&2
		return 1
	fi

	rm -f /tmp/alertsys_out.$$
	return 0
}


###########################################################################
#
# AlertSys_load_pending()
#
#   Sets vars: pcnt, PendingTime, LastAlert
#
###########################################################################
AlertSys_load_pending() {
	if ! which curl >/dev/null 2>/dev/null
	then
		echo "AlertSys_load_pending(): curl not found" >&2
		return 2
	fi

	curl --silent --show-error \
		--output /tmp/alertsys_pending.$$ \
		--url "$ALERTSYS_BASE/alerts/pending/"

	if [ $? -ne 0 ]
	then
		echo "AlertSys_load_pending(): curl command failed" >&2
		return 1
	fi

	pcnt=`sed -n 1p /tmp/alertsys_pending.$$`
	PendingTime=`sed -n 2p /tmp/alertsys_pending.$$`
	LastAlert=`sed -n 3p /tmp/alertsys_pending.$$`

	if echo "$pcnt" | grep -q '^[0-9]\+$'; then
		# Looks OK
		rm -f /tmp/alertsys_pending.$$
		return 0
	else
		echo "AlertSys_load_pending(): alertsys error (see /tmp/alertsys_pending.$$)" >&2
		return 1
	fi
}

###########################################################################
#
# AlertSys_load_hostlink()
#
#   Gets the shortened URL for the alertsys UI for this host
#
#   Sets var: hostlink
#
###########################################################################
AlertSys_load_hostlink() {
	if ! which curl >/dev/null 2>/dev/null
	then
		echo "AlertSys_load_hostlink(): curl not found" >&2
		return 2
	fi

	curl --silent --show-error \
		--output /tmp/alertsys_hostlink.$$ \
		--url "$ALERTSYS_BASE/alerts/host_link/"

	if [ $? -ne 0 ]
	then
		echo "AlertSys_load_hostlink(): curl command failed" >&2
		return 1
	fi

	hostlink=`cat /tmp/alertsys_hostlink.$$`

	if echo "$hostlink" | grep -q '^http'; then
		# Looks OK
		rm -f /tmp/alertsys_hostlink.$$
		return 0
	else
		echo "AlertSys_load_hostlink(): alertsys error (see /tmp/alertsys_hostlink.$$)" >&2
		return 1
	fi
}

###########################################################################
#
# AlertSys_reset()
#
#   Resolve all alerts from this host
#
###########################################################################
AlertSys_reset() {
	if ! which curl >/dev/null 2>/dev/null
	then
		echo "AlertSys_reset(): curl not found" >&2
		return 2
	fi

	curl --silent --show-error \
		--output /tmp/alertsys_reset.$$ \
		--url "$ALERTSYS_BASE/alerts/reset/"

	if [ $? -ne 0 ]
	then
		echo "AlertSys_reset(): curl command failed" >&2
		return 1
	fi

	tmp_output=`cat /tmp/alertsys_reset.$$`

	if [ "X$tmp_output" = "XOK" ]
	then
		rm -f /tmp/alertsys_reset.$$
		return 0
	else
		echo "AlertSys_reset(): alertsys error (see /tmp/alertsys_reset.$$)" >&2
		return 1
	fi
}

