#ifndef GATIME_H
#define GATIME_H

/**
 * @file gatime.h
 */

#include <sys/time.h>

#ifdef	__cplusplus
extern "C" {
#endif

typedef struct timeval GATIME; //!< @brief gatime

/**
 * @brief  Macros for function calls to fill a GATIME struct with
 * the current time.
 */
#define gatime(tp)		gettimeofday (tp, NULL)		

/**
 * @brief Macro for calculating difference in milliseconds between two
 * GATIME structs.
 */
#define diff_gatimes(nt,ot)		((1000*(nt.tv_sec - ot.tv_sec) + \
								nt.tv_usec/1000) - ot.tv_usec/1000)

/** @brief  Extract seconds from a GATIME struct. */
#define ga_getsecs(t)			(long)t.tv_sec
/** @brief  Extract milliseconds from a GATIME struct. */
#define ga_getmilli(t)			t.tv_usec/1000

/**
 * @brief get dst status
 * In timeutil.c
 * @return
 */
int get_dst_status (void);

/**
 * @brief Convert WAVO local date and time string 12/25/1999"<TAB>"15:04:07
 * @param instr
 * @return
 */
struct tm *time_str_to_gmt (char *instr);

/**
 * @brief Convert WAVO local date and time string 12/25/1999"<TAB>"15:04:07 (re-entry)
 * @param instr
 * @param ptm
 * @return
 */
struct tm *time_str_to_gmt_r (char *instr, struct tm *ptm);

/**
 * @brief time str to time t 
 * @param instr
 * @return
 */
time_t time_str_to_time_t (char *instr);

/**
 * @brief Format date and time in standard ISO notation.
 * @param stddate
 * @param in_time
 * @param fmt_str
 * @return
 */
char *iso_time_fmt (char *stddate, char *in_time, char *fmt_str);

/**
 * @brief Format date and time in standard ISO notation. (re-entry)
 * @param stddate
 * @param in_time
 * @param fmt_str
 * @param out_time
 * @param out_len
 * @return
 */
char *iso_time_fmt_r (char *stddate, char *in_time, char *fmt_str, \
                      char *out_time, int out_len);
/**
 * @brief iso time 
 * @param stddate
 * @param in_time
 * @return
 */
char *iso_time (char *stddate, char *in_time);

/**
 * @brief iso time (re-entry)
 * @param stddate
 * @param in_time
 * @param out_str
 * @param out_len
 * @return
 */
char *iso_time_r (char *stddate, char *in_time, char *out_str, int out_len);

/** 
 * @brief Convert passed in iso time to local time equivalent.
 * @param in_iso
 * @return 
 */
char *shift_iso_time (char *in_iso);

/**
 * @brief Convert passed in iso time to local time equivalent. (re-entry)
 * @param in_iso
 * @param out_iso
 * @param out_len
 * @return
 */
char *shift_iso_time_r (char *in_iso, char *out_iso, int out_len);

/**
 * @brief set is time log level
 * @param loglvl
 */
void set_iso_time_loglvl (int loglvl);

/**
 * @brief iso to local std date time
 * @param in_iso
 * @return
 */
char *iso_to_local_std_date_time (char *in_iso);

/**
 * @brief iso to local std date time (re-entry)
 * @param in_iso
 * @param datime
 * @param datime_len
 * @return
 */
char *iso_to_local_std_date_time_r (char *in_iso, int keep_seconds, 
				    char *datime, int datime_len);

/**
 * @brief iso to local std date time 
 * @param in_iso
 * @param keep_seconds
 * @return
 */
char *iso_to_local_std_date_time_2 (char *in_iso, int keep_seconds);

/**
 * @brief iso to local std date time (re-entry)
 * @param in_iso
 * @param keep_seconds
 * @param date_time
 * @param datime_len
 * @return
char *iso_to_local_std_date_time_2_r(char *in_iso, int keep_seconds, \
                                     char *date_time, int datime_len);
*/

/**
 * @brief time util format seconds
 * @param in_seconds
 * @param outstr
 */
void timeutil_format_seconds (long in_seconds, char *outstr);

#ifdef	__cplusplus
}
#endif

#endif		/* GATIME_H */
